package PureMessage::Policy::Blocklist;

use strict;
use base 'PureMessage::Policy::Plugin';
use PureMessage::Config qw(%config);
use Carp qw(croak);
use PureMessage::Logger;

use IO::Select;
use IO::Socket::INET6;
use Socket qw(AF_INET AF_INET6);

our $VERSION = '6.000';

# Logging
our $msg_logger = PureMessage::Logger->new(ident => 'pmx_blocklist');
sub logger {
    my ($level, $msg) = @_;
    $msg_logger->log($level, $msg);
}

my ($host, $port, $protocol) = ('localhost', 4466, 'inet');
if(my $token = $config{blocklist}{Policy}{port}) {
    if($token =~ /(^[^:]+):([^@]+)@(.+)$/) {
        $protocol = $1;
        $port = $2;
        $host = $3;
    }
}

sub commands {
    my ($cls, undef, $logger) = @_;
    my %cmds;

    local $msg_logger = $logger if defined $logger;
    %cmds = (
	'pmx_blocklist' => \&pmx_blocklist,
    );
    %cmds;
}

sub pmx_blocklist {
    my ($msg, $args) = @_;
    my ($cmp,$result) = (0,"ok");

    $msg->assert_event('eom');
    my $client_ip = $msg->minfo_value("Fur");
    my $rdns = $msg->minfo_value("Fur_rdns"); #only PMX sets this
    $rdns ||= 'unknown';
    my $helo = $msg->minfo_value("Fur_helo");
    $helo ||= 'unknown';
    my $auth = $msg->minfo_value('Fur_auth');
    $auth ||= 0;

    # SEA has an minfo_value Fur of: "<rdns|ip> [<ip>]"
    if ($client_ip =~ m/(\S+) \[(.*)\]/) {
        $rdns = $1;
        $client_ip = $2;
    }
    $msg->log('DEBUG', "fur: IP=$client_ip RDNS=$rdns HELO=$helo auth=$auth");
    if($client_ip ne 'none' and !$auth) {
        if (my @resp = _lookup($msg, $client_ip, $rdns, $helo)) {
            if (scalar @resp == 3) {
                # Three values - rcode,xcode & message
                $msg->template('BLOCKLIST_REASON', $resp[2]);
                $result = "reject";
                $cmp = 1;
            }
        } else {
            $msg->log('ERROR', "pmx_blocklist: timeout or error querying the blocker daemon");
        }
    }
    $msg->set_attr('mark_b',$result);
    $cmp;
}

sub _lookup {
    my ($msg, $sender_ip, $rdns, $helo) = @_;

    my $domain = (lc $protocol eq 'inet') ? AF_INET : AF_INET6;
    my $sock = new IO::Socket::INET6 (
        PeerAddr => $host,
        PeerPort => $port,
        Domain   => $domain,
        Proto    => 'tcp',
    );
    if (!$sock) {
        $msg->log('ERROR', "pmx_blocklist: error connecting to blocker daemon");
        return;
    }

    my $data =  <<"EOT";
request=puremessage_policy
protocol_state=DATA
helo_name=$helo
client_address=$sender_ip
reverse_client_name=$rdns

EOT
    my $len_written = $sock->syswrite($data);
    if ($len_written != length($data)) {
        $msg->log('ERROR', "pmx_blocklist: error writing to socket: $!");
        $sock->close;
        return;
    }

    my @response;
    my $buffer;
    my $accum = '';

    my $listener = IO::Select->new($sock);
    if(!$listener) {
        $msg->log('ERROR', "pmx_blocklist: error creating timeout monitor");
        $sock->close;
        return;
    }

    while($listener->can_read(1) > 0) {
        my $bytes = $sock->sysread($buffer, 1024);
        if ($bytes <= 0) {
            $msg->log('ERROR',
                "pmx_blocklist: error reading from blocker daemon socket: $!");
            last;
        }
        $accum .= $buffer;
        if ($accum =~ m/action=(\d+)\s+(\S+)\s+(.+)\n\n/) {
            @response = ($1, $2, $3);
            last;
        } elsif ($accum =~ m/action=(.+)\n\n/) {
            @response = ($1);
            last;
        }
    }

    $sock->close or
        $msg->log('ERROR', "pmx_blocklist: error closing socket: $!");
    return @response;
}

1;
